@@ -28,6 +28,7 @@ class FirewallDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     POLICY_DROP = 1
 
     ALL_NODES = "all"
+    ALL_NODES_IDX = 0
 
     _notification_callback = QtCore.pyqtSignal(str, ui_pb2.NotificationReply)
 
@@ -107,6 +108,7 @@ class FirewallDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     @QtCore.pyqtSlot(int)
     def _cb_nodes_updated(self, total):
         if self._nodes.count() <= 1:
+            self._load_nodes()
             self.load_fw_policies()
 
     def _cb_combo_nodes_changed(self, idx):
@@ -134,6 +136,14 @@ class FirewallDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 self._set_status_error(QC.translate("firewall", "error adding profile extra rules:", err))
 
     def _cb_combo_policy_changed(self, combo):
+        if self.comboNodes.currentIndex() == FirewallDialog.ALL_NODES_IDX:
+            ret = Message.yes_no(
+                QC.translate("stats", "This change will apply the policy change to all nodes?"),
+                QC.translate("stats", "Are you sure?"),
+                QtWidgets.QMessageBox.Icon.Warning)
+            if ret == QtWidgets.QMessageBox.StandardButton.Cancel:
+                return
+
         self._reset_status_message()
         self.comboInput.setEnabled(False)
         self.comboOutput.setEnabled(False)
@@ -273,7 +283,8 @@ class FirewallDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 if self._nodes.count() == 1:
                     nIdx = self.comboNodes.currentIndex()
                     node_addr = self.comboNodes.itemData(nIdx)
-                    enableFw = self.load_node_fw_policy(node_addr)
+                    if node_addr is not None:
+                        enableFw = self.load_node_fw_policy(node_addr)
                     return
 
         except Exception as e:
