@@ -16,7 +16,7 @@ import opensnitch.proto as pb2
 ui_pb2, ui_pb2_grpc = pb2.import_()
 
 from opensnitch.dialogs.prompt import PromptDialog
-from opensnitch.dialogs.stats import StatsDialog
+from opensnitch.dialogs.events import StatsDialog
 
 from opensnitch.plugins import PluginsManager, PluginBase, PluginsList, PluginSignal
 from opensnitch.actions import Actions
@@ -31,7 +31,13 @@ from opensnitch.utils import (
     CleanerTask,
     OneshotTimer,
     languages,
-    Message
+    Message,
+    logger
+)
+from opensnitch.proto.enums import (
+    ConnFields,
+    NodeFields,
+    RuleFields
 )
 from opensnitch.utils.duration import duration
 from opensnitch.utils.themes import Themes
@@ -59,7 +65,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self.MENU_ENTRY_FW_ENABLE = QtCore.QCoreApplication.translate("contextual_menu", "Enable")
         self.MENU_ENTRY_FW_DISABLE = QtCore.QCoreApplication.translate("contextual_menu", "Disable")
         self.MENU_ENTRY_HELP = QtCore.QCoreApplication.translate("contextual_menu", "Help")
-        self.MENU_ENTRY_CLOSE = QtCore.QCoreApplication.translate("contextual_menu", "Close")
+        self.MENU_ENTRY_QUIT = QtCore.QCoreApplication.translate("contextual_menu", "Quit")
 
         # set of actions that must be performed on the main thread
         self.NODE_ADD = 0
@@ -68,6 +74,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self.ADD_RULE = 3
         self.DELETE_RULE = 4
 
+        self.logger = logger.get(__name__)
         self._cfg = Config.init()
         self._db = Database.instance()
         db_file=self._cfg.getSettings(self._cfg.DEFAULT_DB_FILE_KEY)
@@ -162,7 +169,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                     action = action_def['actions'][name]
                     action.run()
                 except Exception as e:
-                    print("global._load_plugins() exception:", e, "is {0} enabled?".format(name))
+                    self.logger.warning("global._load_plugins() exception: %s - is %s enabled?", repr(e), name)
 
     def _stop_plugins(self):
         for plug in PluginsList.actions:
@@ -240,7 +247,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self._menu.addAction(self.MENU_ENTRY_HELP).triggered.connect(
                 lambda: QtGui.QDesktopServices.openUrl(QtCore.QUrl(Config.HELP_CONFIG_URL))
                 )
-        self._menu.addAction(self.MENU_ENTRY_CLOSE).triggered.connect(self._on_close)
+        self._menu.addAction(self.MENU_ENTRY_QUIT).triggered.connect(self._on_close)
 
         self._menu.aboutToShow.connect(self._on_show_menu)
 
@@ -248,10 +255,14 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         try:
             self._autostart.enable(self._menu_autostart.isChecked())
         except Exception as e:
-            self._desktop_notifications.show(
-                QC.translate("stats", "Warning"),
-                QC.translate("stats", str(e))
-            )
+            has_ntfs, ntfs_type = self._has_desktop_notifications()
+            if has_ntfs:
+                self.show_systray_msg(
+                    QC.translate("stats", "Warning"),
+                    QC.translate("stats", "Error switching autostart: {0}".format(str(e))),
+                    mtype=ntfs_type
+                )
+            self.logger.warning("Error switching autostart: %s", repr(e))
 
     def _on_show_menu(self):
         self._menu_autostart.setChecked(self._autostart.isEnabled())
@@ -269,24 +280,49 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
         QtCore.QTimer.singleShot(10000, __show_gui)
 
-    def _show_systray_msg_error(self):
-        print("")
-        print("WARNING: system tray not available. On GNOME you need the extension gnome-shell-extension-appindicator.")
-        print("\tRead more:", Config.HELP_SYSTRAY_WARN)
-        print("\tIf you want to start OpenSnitch GUI in background even if tray not available, use --background argument.")
-        print("")
+    def show_systray_msg(self, title, body, icon=None, callback=None, user_args=None, mtype=Config.NOTIFICATION_TYPE_QT, timeout=10):
+        try:
+            if mtype == Config.NOTIFICATION_TYPE_QT:
+                if not isinstance(icon, QtWidgets.QSystemTrayIcon.MessageIcon):
+                    icon = QtWidgets.QSystemTrayIcon.MessageIcon.Information
+                self._tray.showMessage(title, body, icon, timeout * 1000)
+            else:
+                if icon is None:
+                    icon = "dialog-information"
+                self._desktop_notifications.show(
+                    title,
+                    body,
+                    icon,
+                    callback=callback
+                )
+        except Exception as e:
+            self.logger.warning("error showing notification: %s", repr(e))
+            self.logger.warning(f"{title}, {body}")
 
-        hide_msg = self._cfg.getBool(Config.DEFAULT_HIDE_SYSTRAY_WARN)
-        if hide_msg:
-            return
-        self._desktop_notifications.show(
-            QC.translate("stats", "WARNING"),
-            QC.translate("stats", """System tray not available. Read more:
+    def _show_systray_msg_error(self):
+        try:
+            print("")
+            print("WARNING: system tray not available. On GNOME you need the extension gnome-shell-extension-appindicator.")
+            print("\tRead more:", Config.HELP_SYSTRAY_WARN)
+            print("\tIf you want to start OpenSnitch GUI in background even if tray not available, use --background argument.")
+            print("")
+
+            hide_msg = self._cfg.getBool(Config.DEFAULT_HIDE_SYSTRAY_WARN)
+            if hide_msg:
+                return
+            has_ntfs, ntf_type = self._has_desktop_notifications()
+            if has_ntfs:
+                self.show_systray_msg(
+                    QC.translate("stats", "WARNING"),
+                    QC.translate("stats", """System tray not available. Read more:
 {0}
 """.format(Config.HELP_SYSTRAY_WARN)),
-            os.path.join(self._path, "res/icon-white.svg")
-        )
-        self._cfg.setSettings(Config.DEFAULT_HIDE_SYSTRAY_WARN, True)
+                    icon=os.path.join(self._path, "res/icon-white.svg"),
+                    mtype=ntf_type
+                )
+                self._cfg.setSettings(Config.DEFAULT_HIDE_SYSTRAY_WARN, True)
+        except Exception as e:
+            self.logger.warning("exception showing systray msg error: %s", repr(e))
 
     def _on_tray_icon_activated(self, reason):
         if reason == QtWidgets.QSystemTrayIcon.ActivationReason.Trigger or reason == QtWidgets.QSystemTrayIcon.ActivationReason.MiddleClick:
@@ -395,7 +431,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
 
         except Exception as e:
-            print("PostAlert() exception:", e)
+            self.logger.warning("PostAlert() exception: %s", repr(e))
             return ui_pb2.MsgResponse(id=1)
 
     @QtCore.pyqtSlot(str, ui_pb2.PingRequest)
@@ -421,12 +457,17 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
     @QtCore.pyqtSlot(str, ui_pb2.NotificationReply)
     def _on_notification_reply(self, addr, reply):
         if reply.code == ui_pb2.ERROR:
-            self._tray.showMessage(
-                "Error",
-                reply.data,
-                QtWidgets.QSystemTrayIcon.MessageIcon.Information,
-                5000
-            )
+            has_ntfs, ntf_type = self._has_desktop_notifications()
+            if has_ntfs:
+                self.show_systray_msg(
+                    "Error",
+                    reply.data,
+                    icon=os.path.join(self._path, "res/icon-white.svg"),
+                    mtype=ntf_type,
+                    timeout=5000
+                )
+            else:
+                print("[service] notification reply error:", addr, reply.data)
 
     def _on_remote_stats_menu(self, address):
         self._remote_stats[address]['dialog'].show()
@@ -439,21 +480,22 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 #self._stats_dialog.raise()
                 self._stats_dialog.activateWindow()
 
-        if self._desktop_notifications.are_enabled():
+        has_ntfs, ntf_type = self._has_desktop_notifications()
+        if has_ntfs:
             timeout = self._cfg.getInt(Config.DEFAULT_TIMEOUT_KEY, 15)
-
-            if self._desktop_notifications.is_available() and self._cfg.getInt(Config.NOTIFICATIONS_TYPE, 1) == Config.NOTIFICATION_TYPE_SYSTEM:
-                try:
-                    self._desktop_notifications.show(
-                        title,
-                        body,
-                        os.path.join(self._path, "res/icon-white.svg"),
-                        callback=callback_open_clicked
-                    )
-                except:
-                    self._tray.showMessage(title, body, icon, timeout * 1000)
-            else:
+            try:
+                self.show_systray_msg(
+                    title,
+                    body,
+                    icon=os.path.join(self._path, "res/icon-white.svg"),
+                    callback=callback_open_clicked,
+                    mtype=ntf_type,
+                    timeout=timeout
+                )
+            except:
                 self._tray.showMessage(title, body, icon, timeout * 1000)
+        else:
+            self._tray.showMessage(title, body, icon, timeout * 1000)
 
         if icon == QtWidgets.QSystemTrayIcon.MessageIcon.NoIcon:
             self._tray.setIcon(self.alert_icon)
@@ -474,6 +516,14 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         if theme_idx > 0:
             self._themes.load_theme(self._app)
 
+    def _has_desktop_notifications(self):
+        desk_ntfs_available = self._desktop_notifications.is_available() and self._desktop_notifications.are_enabled()
+        ntf_type = Config.NOTIFICATION_TYPE_QT
+        if desk_ntfs_available:
+            ntf_type = Config.NOTIFICATION_TYPE_SYSTEM
+        ntf_type = self._cfg.getInt(Config.NOTIFICATIONS_TYPE, ntf_type)
+        return self._tray.isSystemTrayAvailable() or desk_ntfs_available, ntf_type
+
     def _init_translation(self):
         if self.translator:
             self._app.removeTranslator(self.translator)
@@ -547,12 +597,17 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         if self._connected == False:
             return
         if self._nodes.count() == 0:
-            self._tray.showMessage(
-                QC.translated("stats", "No nodes connected"),
-                "",
-                QtWidgets.QSystemTrayIcon.MessageIcon.Information,
-                5000
-            )
+            has_ntfs, ntfs_type = self._has_desktop_notifications()
+            if has_ntfs:
+                self.show_systray_msg(
+                    QC.translated("stats", "No nodes connected"),
+                    "",
+                    icon=QtWidgets.QSystemTrayIcon.MessageIcon.Information,
+                    mtype=ntfs_type,
+                    timeout=5000
+                )
+            else:
+                print("[service] enable_interception: no nodes connected")
             return
         if self._nodes.count() > 1:
             print("enable interception for all nodes not supported yet")
@@ -578,6 +633,50 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
         return False
 
+    def _build_missed_rule_msg(self, conn, rule, node, hostname):
+        try:
+            _title = conn.process_path
+            if _title == "":
+                _title = "%s:%d (%s)" % (conn.dst_host if conn.dst_host != "" else conn.dst_ip, conn.dst_port, conn.protocol)
+
+            tmpl = self._cfg.getSettings(Config.NOTIFICATIONS_MISSED_POPUP_TMPL, Config.NTF_DEFAULT_MISSED_POPUP_TMPL)
+
+            if f'%{ConnFields.SrcPort.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.SrcPort.value}%", conn.src_port)
+            if f'%{ConnFields.SrcIP.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.SrcIP.value}%", conn.src_ip)
+            if f'%{ConnFields.DstHost.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.DstHost.value}%", conn.dst_host)
+            if f'%{ConnFields.DstIP.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.DstIP.value}%", conn.dst_ip)
+            if f'%{ConnFields.DstPort.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.DstPort.value}%", conn.dst_port)
+            if f'%{ConnFields.Proto.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.Proto.value}%", conn.protocol)
+            if f'%{ConnFields.Process.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.Process.value}%", conn.process)
+            if f'%{ConnFields.ProcCWD.value}%' in tmpl and conn.process_cwd != "":
+                    tmpl = tmpl.replace(f"%{ConnFields.ProcCWD.value}%", conn.process_cwd)
+            if f'%{ConnFields.Cmdline.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.Cmdline.value}%", ' '.join(conn.process_args))
+            if f'%{RuleFields.Action.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{RuleFields.Action.value}%", rule.action)
+            if f'%{ConnFields.Action.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{ConnFields.Action.value}%", rule.action)
+            if f'%{NodeFields.Addr.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{NodeFields.Addr.value}%", node)
+            if f'%{NodeFields.Hostname.value}%' in tmpl:
+                tmpl = tmpl.replace(f"%{NodeFields.Hostname.value}%", hostname)
+
+        except Exception as e:
+            self.logger.warning("_build_missed_rule_msg() exception: %s", repr(e))
+        finally:
+            if tmpl == "":
+                tmpl = "{0} action applied {1}\nProcess: {2}".format(
+                    rule.action, node, " ".join(conn.process_args)
+                )
+
+        return _title, tmpl
     def _get_peer(self, peer):
         """
         server          -> client
@@ -600,7 +699,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             self._nodes.delete(peer)
             self._stats_dialog.update(True, None, True)
         except Exception as e:
-            print("_delete_node() exception:", e)
+            self.logger.warning("_delete_node() exception: %s", repr(e))
 
     def _populate_stats(self, db, proto, addr, stats):
         main_need_refresh = False
@@ -655,7 +754,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             for event in stats.events:
                 self._last_stats[addr].append(event.unixnano)
         except Exception as e:
-            print("_populate_stats() exception: ", e)
+            self.logger.warning("_populate_stats() exception: %s", repr(e))
 
         return main_need_refresh, details_need_refresh
 
@@ -702,7 +801,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
             self._last_items[table][addr] = items
         except Exception as e:
-            print("details exception: ", e)
+            self.logger.warning("populate_stats details exception: %s", repr(e))
 
         return need_refresh
 
@@ -719,11 +818,11 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             else:
                 temp_cfg['DefaultAction'] = Config.ACTION_DENY
 
-            print("Setting daemon DefaultAction to:", temp_cfg['DefaultAction'])
+            self.logger.info("Setting daemon DefaultAction to: %s", temp_cfg['DefaultAction'])
 
             newconf.config = json.dumps(temp_cfg)
         except Exception as e:
-            print("error parsing node's configuration:", e)
+            self.logger.warning("error parsing node's configuration: %s", repr(e))
             return node_config
 
         return newconf
@@ -820,7 +919,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             #            self._remote_stats[addr]['dialog'].update(addr, request.stats)
 
         except Exception as e:
-            print("Ping exception", context.peer(), ":", e)
+            self.logger.warning("exception %s - %s", repr(context.peer()), repr(e))
         return ui_pb2.PingReply(id=request.id)
 
     def AskRule(self, request, context):
@@ -834,6 +933,7 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         self._asking = True
         peer = context.peer()
         proto, addr = self._get_peer(peer)
+        hostname = self._nodes.get_node_hostname("%s:%s" % (proto, addr))
         rule, timeout_triggered = self._prompt_dialog.promptUser(request, self._is_local_request(proto, addr), peer)
         self._last_ping = datetime.now()
         self._asking = False
@@ -841,16 +941,11 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             return None
 
         if timeout_triggered:
-            _title = request.process_path
-            if _title == "":
-                _title = "%s:%d (%s)" % (request.dst_host if request.dst_host != "" else request.dst_ip, request.dst_port, request.protocol)
-
-
             node_text = "" if self._is_local_request(proto, addr) else "on node {0}:{1}".format(proto, addr)
+            _title, _msg = self._build_missed_rule_msg(request, rule, node_text, hostname)
             self._show_message_trigger.emit(
                 _title,
-                "{0} action applied {1}\nCommand line: {2}"
-                .format(rule.action, node_text, " ".join(request.process_args)),
+                _msg,
                 QtWidgets.QSystemTrayIcon.MessageIcon.NoIcon,
                 DesktopNotifications.URGENCY_NORMAL
             )
@@ -881,10 +976,11 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
         @doc: https://grpc.github.io/grpc/python/grpc.html#service-side-context
         """
-        print(datetime.now(), "[Subscribe]", context.peer())
+        self.logger.info("%s", repr(context.peer()))
         # if the exit mark is set, don't accept new connections.
         # db vacuum operation may take a lot of time to complete.
         if self._exit:
+            self.logger.info("_exit mark set, exiting, %s", repr(context.peer()))
             context.cancel()
             return
         try:
@@ -899,14 +995,19 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
 
             proto, addr = self._get_peer(context.peer())
             if self._is_local_request(proto, addr) == False:
+                hostname = self._nodes.get_node_hostname("%s:%s" % (proto, addr))
+                body = "{0}".format(context.peer())
+                if hostname != "":
+                    body = "{0}\n{1}".format(hostname, context.peer())
+
                 self._show_message_trigger.emit(
                     QtCore.QCoreApplication.translate("stats", "New node connected"),
-                    "({0})".format(context.peer()),
+                    body,
                     QtWidgets.QSystemTrayIcon.MessageIcon.Information,
                     DesktopNotifications.URGENCY_LOW
                 )
         except Exception as e:
-            print("[Notifications] exception adding new node:", e)
+            self.logger.warning("exception adding new node: %s", repr(e))
             context.cancel()
 
         newconf = self._overwrite_nodes_config(node_config)
@@ -922,22 +1023,33 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
         @doc: https://grpc.io/docs/what-is-grpc/core-concepts/
         """
         local_peer = context.peer()
+        self.logger.info("channel started: %s", repr(local_peer))
 
-        print(datetime.now(), "[Notifications] channel started:", context.peer())
-        print()
         proto, addr = self._get_peer(context.peer())
         node_addr = f"{proto}:{addr}"
-        _node = self._nodes.get_node(node_addr)
-        if _node == None:
+        cur_node = self._nodes.get_node(node_addr)
+        if cur_node is None:
             return
 
         stop_event = Event()
         def _on_client_closed():
+            self.logger.info("client closed %s, now: %s", local_peer, datetime.now())
+
+            # Get latest node info of this address.
+            prev_node = self._nodes.get_node(node_addr)
+            # If the peer of this notification is not the same of the current node,
+            # don't update the status.
+            # This can occur, if a node disconnects by timeout and reconnects
+            # after some minutes or hours.
+            # The node will connect with a new peer, but the old one still
+            # exists in the server. So when the server decides to close that
+            # inactive session, it'll report the old connection.
+            if prev_node is not None:
+                if prev_node['session']['peer'] != local_peer and cur_node['last_seen'] > cur_node['session']['last_seen']:
+                    return
+
             stop_event.set()
-            print(datetime.now(), "[Notifications] client closed", context.peer())
             self._nodes.stop_notifications(node_addr)
-
-
             self._node_actions_trigger.emit(
                 {'action': self.NODE_DELETE,
                  'peer': local_peer,
@@ -950,21 +1062,26 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
             #    nd = self._nodes.get_nodes()
             #    if nd[0].get_config().isFirewallRunning:
 
-            if self._is_local_request(proto, addr) == False:
+            if not self._is_local_request(proto, addr):
+                self.logger.info("notifying exit %s", node_addr)
+                hostname = self._nodes.get_node_hostname(node_addr)
+                body = "{0}".format(local_peer)
+                if hostname != "":
+                    body = "{0}\n{1}".format(hostname, local_peer)
                 self._show_message_trigger.emit(
                     "node exited",
-                    f"({local_peer})",
+                    body,
                     QtWidgets.QSystemTrayIcon.MessageIcon.Information,
                     DesktopNotifications.URGENCY_LOW
                 )
 
         added = context.add_callback(_on_client_closed)
         if added is False:
-            print("[Notifications] add_callback() not added")
+            self.logger.debug("add_callback() not added")
 
         # TODO: move to notifications.py
         def new_node_message():
-            print("new node connected, listening for client responses...", addr)
+            self.logger.info("new node connected, listening for client responses... %s", repr(local_peer))
 
             while self._exit == False:
                 try:
@@ -972,18 +1089,18 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                         break
 
                     in_message = next(node_iter)
-                    if in_message == None:
+                    if in_message is None:
                         continue
 
                     self._nodes.reply_notification(addr, in_message)
                 except StopIteration:
-                    print("[Notifications] Node {0} exited".format(addr))
+                    self.logger.info("Node %s exited", node_addr)
                     break
                 except grpc.RpcError as e:
-                    print(datetime.now(), "[Notifications] grpc exception new_node_message(): ", addr, e)
+                    self.logger.info("grpc exception new_node_message(): %s - %s", node_addr, repr(e))
                     break
                 except Exception as e:
-                    print("[Notifications] unexpected exception new_node_message(): ", addr, e)
+                    self.logger.warning("unexpected exception new_node_message() %s: %s", node_addr, repr(e))
                     break
 
         read_thread = Thread(target=new_node_message)
@@ -995,19 +1112,20 @@ class UIService(ui_pb2_grpc.UIServicer, QtWidgets.QGraphicsObject):
                 break
 
             try:
-                noti = _node['notifications'].get()
+                noti = cur_node['notifications'].get()
                 if noti is not None:
                     if noti.type > 0:
-                        _node['notifications'].task_done()
+                        self.logger.debug("%s delivering notification... %s", node_addr, repr(noti))
+                        cur_node['notifications'].task_done()
                         yield noti
                     elif noti.type == -1:
-                        print("[Notifications] notify exit, break the loop")
+                        self.logger.debug("%s notify exit, break the loop", node_addr)
                         break
             except Exception as e:
-                print("[Notifications] exception getting notification from queue:", addr, e)
+                self.logger.warning("exception getting notification from queue: %s - %s", node_addr, repr(e))
                 break
 
         # force call to _on_client_closed
         context.cancel()
-        print("[Notifications] channel closed:", local_peer)
+        self.logger.info("channel closed: %s", repr(local_peer))
         return node_iter
